--*************************************************************************
--  CREACIO TAULA VOTACIO PER EMAGATZEMAR LES VOTACIONS PUBLICADES
--*************************************************************************

DROP TABLE votacio CASCADE CONSTRAINTS PURGE;

CREATE TABLE votacio (
    idVotacio           INTEGER       CONSTRAINT PK_votacio PRIMARY KEY,
    titol               VARCHAR2(50)  CONSTRAINT NN_votacioTitol NOT NULL,
    descripcio          VARCHAR2(150) ,
    dataInici           DATE          CONSTRAINT NN_votacioDataInici NOT NULL,
    dataFinal           DATE          CONSTRAINT NN_votacioDataFinal NOT NULL,
    dataPublicacio      DATE          CONSTRAINT NN_votacioDataPublicacio NOT NULL,
    presidentMesa       VARCHAR2(80)  CONSTRAINT NN_votacioPresidentMesa NOT NULL,
    urlWeb              VARCHAR2(150),
    estat               VARCHAR2(1)   CONSTRAINT NN_votacioEstat NOT NULL
                                      CONSTRAINT CH_votacioEstat
                                          CHECK( estat IN ('E', 'P', 'O', 'F')),
    nPersonesDretVot    NUMBER        CONSTRAINT CH_votacioNPersonesDretVot CHECK( nPersonesDretVot >=0 ),
    nVotsTotalsEmesos   NUMBER        CONSTRAINT CH_votacioNVotsTotalsEmesos CHECK( nVotsTotalsEmesos >=0 ),
    percenParticipacio  NUMBER(5,2)   CONSTRAINT CH_votacioPercenParticipacio 
                                          CHECK( percenParticipacio BETWEEN 0 AND 100)
    );
CREATE UNIQUE INDEX votacio_titol on votacio (titol);

DROP SEQUENCE seq_votacio;

CREATE SEQUENCE seq_votacio 
      INCREMENT BY 1 
      START WITH 1;

--  Creacio disparador tr_idVotacio_insert per autonumerar les votacions
CREATE OR REPLACE TRIGGER tr_IdVotacio_insert
      BEFORE INSERT ON votacio
      FOR EACH ROW
      BEGIN
          SELECT seq_votacio.NEXTVAL INTO :NEW.idVotacio
          FROM DUAL;
      END tr_IdVotacio_insert;
      
/

--*************************************************************************
--  CREACIO TAULA OPCIO PER EMAGATZEMAR LES OPCIONS DISPONIBLES PER VOTAR
--*************************************************************************

DROP TABLE opcio CASCADE CONSTRAINTS PURGE;

CREATE TABLE opcio (
    idOpcio INTEGER       CONSTRAINT PK_opcio PRIMARY KEY,
    nom     VARCHAR2(50)  CONSTRAINT NN_opcioNom NOT NULL
    );
CREATE UNIQUE INDEX opcio_nom on opcio (nom);

DROP SEQUENCE seq_opcio;

CREATE SEQUENCE seq_opcio 
      INCREMENT BY 1 
      START WITH 1;

--  Creacio disparador tr_idOpcio_insert per autonumerar les opcionc
CREATE OR REPLACE TRIGGER tr_IdOpcio_insert
      BEFORE INSERT ON opcio
      FOR EACH ROW
      BEGIN
          SELECT seq_opcio.NEXTVAL INTO :NEW.idOpcio
          FROM DUAL;
      END tr_IdOpcio_insert;
      
/

--*************************************************************************
--  CREACIO TAULA PAIS PER EMAGATZEMAR ELS PAISOS DE LA COMUNITAT EUROPEA
--*************************************************************************

DROP TABLE pais CASCADE CONSTRAINTS PURGE;

CREATE TABLE pais (
    idPais  CHAR(2)       CONSTRAINT PK_pais PRIMARY KEY,
    nom     VARCHAR2(50)  CONSTRAINT NN_paisNom NOT NULL
    );
CREATE UNIQUE INDEX pais_nom on pais (nom);

--*************************************************************************
--  CREACIO TAULA REGIO PER EMAGATZEMAR LES REGIONS DELS PAISOS DE LA COMUNITAT EUROPEA
--*************************************************************************

DROP TABLE regio CASCADE CONSTRAINTS PURGE;

CREATE TABLE regio (
    idPais  CHAR(2)       CONSTRAINT FK1_regioPais REFERENCES pais(idPais),
    idRegio VARCHAR2(10)  CONSTRAINT NN_regioRegio NOT NULL,
    nom     VARCHAR2(100) CONSTRAINT NN_regioNom NOT NULL,
    CONSTRAINT PK_regio PRIMARY KEY(idPais, idRegio)
    );
CREATE UNIQUE INDEX regio_nom on regio (idPais,nom);

--*************************************************************************
--  CREACIO TAULA LOCALITAT PER EMAGATZEMAR LES LOCALITATS DE LA COMUNITAT EUROPEA
--*************************************************************************

DROP TABLE localitat CASCADE CONSTRAINTS PURGE;

CREATE TABLE localitat (
    idPais      CHAR(2)       CONSTRAINT FK1_localitatPais REFERENCES pais(idPais),
    idRegio     VARCHAR2(10)  CONSTRAINT NN_localitatRegio NOT NULL,
    idLocalitat VARCHAR2(10)  CONSTRAINT NN_localitatLocalitat NOT NULL,
    nom         VARCHAR2(100) CONSTRAINT NN_localitatNom NOT NULL,
    CONSTRAINT PK_localitat PRIMARY KEY(idPais, idRegio, idLocalitat),
    CONSTRAINT FK2_localitatRegio FOREIGN KEY (idPais, idRegio) REFERENCES regio(idPais, idRegio)
    );
CREATE INDEX localitat_nom on localitat (idPais, idRegio, nom);

--*************************************************************************
--  CREACIO TAULA CIUTADA PER EMAGATZEMAR LA REFERENCIA A TOTS ELS CIUTADANS DE LA COMUNITAT EUROPEA
--*************************************************************************

DROP TABLE ciutada CASCADE CONSTRAINTS PURGE;

CREATE TABLE ciutada (
    idCiutada     CHAR(32)      CONSTRAINT PK_ciutada PRIMARY KEY,
    idPais        CHAR(2)       CONSTRAINT FK1_ciutadaPais REFERENCES pais(idPais),
    idRegio       VARCHAR2(10)  CONSTRAINT NN_ciutadaRegio NOT NULL,
    idLocalitat   VARCHAR2(10)  CONSTRAINT NN_ciutadaLocalitat NOT NULL,
    dataNaixement DATE          CONSTRAINT NN_ciutadaDataNaixement NOT NULL,
    CONSTRAINT FK2_ciutadaRegio FOREIGN KEY (idPais, idRegio) REFERENCES regio(idPais, idRegio),
    CONSTRAINT FK3_ciutadaLocalitat FOREIGN KEY (idPais, idRegio, idLocalitat) REFERENCES localitat(idPais, idRegio, idLocalitat)
    );

--*************************************************************************
--  CREACIO TAULA CENS PER EMAGATZEMAR ENS CENS DE POBLACIO DE LA COMUNITAT EUROPEA
--*************************************************************************

DROP TABLE cens CASCADE CONSTRAINTS PURGE;

CREATE TABLE cens (
    idCens        NUMBER        CONSTRAINT PK_cens PRIMARY KEY,
    tipus         CHAR(1)       CONSTRAINT NN_censTipus NOT NULL
                                CONSTRAINT CH_censTipus
                                          CHECK( tipus IN ('G', 'E', 'R', 'L')),
    idPais        CHAR(2)       CONSTRAINT FK1_censPais REFERENCES pais(idPais),
    idRegio       VARCHAR2(10)  ,
    idLocalitat   VARCHAR2(10)  ,
    CONSTRAINT FK2_censRegio FOREIGN KEY (idPais, idRegio) REFERENCES regio(idPais, idRegio),
    CONSTRAINT FK3_censLocalitat FOREIGN KEY (idPais, idRegio, idLocalitat) REFERENCES localitat(idPais, idRegio, idLocalitat)
    );
CREATE UNIQUE INDEX cens_tipus on cens (tipus,idPais,idRegio,idLocalitat);
    
DROP SEQUENCE seq_cens;

CREATE SEQUENCE seq_cens 
      INCREMENT BY 1 
      START WITH 1;

--  Creacio disparador tr_idCens_insert per autonumerar els censos
CREATE OR REPLACE TRIGGER tr_IdCens_insert
      BEFORE INSERT ON cens
      FOR EACH ROW
      BEGIN
          SELECT seq_cens.NEXTVAL INTO :NEW.idCens
          FROM DUAL;
      END tr_IdCens_insert;
      
/

--*************************************************************************
--  CREACIO TAULA OPCIONS_VOTACIO PER EMAGATZEMAR LES OPCIONES QUE ES VOTARAN EN CADA VOTACIO
--*************************************************************************

DROP TABLE opcionsVotacio CASCADE CONSTRAINTS PURGE;

CREATE TABLE opcionsVotacio (
    idVotacio   INTEGER     CONSTRAINT FK1_opcionsVotacioIdVotacio REFERENCES votacio(idVotacio) ON DELETE CASCADE,
    idOpcio     INTEGER     CONSTRAINT FK2_opcionsVotacioIdOpcio REFERENCES opcio(idOpcio),
    nNumeroVots NUMBER,
    percenVots  NUMBER(5,2) CONSTRAINT CH_opcionsVotacioPercenVots 
                                  CHECK( percenVots BETWEEN 0 AND 100),
    CONSTRAINT PK_opcionsVotacio PRIMARY KEY(idVotacio, idOpcio)                                  
    );

--*************************************************************************
--  CREACIO TAULA CENS_VOTACIO PER EMAGATZEMAR ELS CENSOS QUE S'HABILITARAN EN CADA VOTACIO
--*************************************************************************

DROP TABLE censVotacio CASCADE CONSTRAINTS PURGE;

CREATE TABLE censVotacio (
    idVotacio   INTEGER     CONSTRAINT FK1_censVotacioIdVotacio REFERENCES votacio(idVotacio) ON DELETE CASCADE,
    idCens      NUMBER      CONSTRAINT FK2_censVotacioIdCens REFERENCES cens(idCens),
    CONSTRAINT PK_censVotacio PRIMARY KEY(idVotacio, idCens)                                  
    );

--*************************************************************************
--  CREACIO TAULA PADRO_CENS PER EMAGATZEMAR ELS CIUTADANS QUE PERTANYEN A CADA CENS
--*************************************************************************

DROP TABLE padroCens CASCADE CONSTRAINTS PURGE;

CREATE TABLE padroCens (
    idCens      NUMBER    CONSTRAINT FK1_padroCensIdCens REFERENCES cens(idCens),
    idCiutada   CHAR(32)  CONSTRAINT FK2_padroCensIdCiutada REFERENCES ciutada(idCiutada) ON DELETE CASCADE,
    CONSTRAINT PK_padroCens PRIMARY KEY(idCens, idCiutada)                                  
    );

--*************************************************************************
--  CREACIO TAULA PARTICIPACIO PER EMAGATZEMAR ELS CIUTADANS QUE HAN VOTAT EN UNA VOTACIO
--*************************************************************************

DROP TABLE participacio CASCADE CONSTRAINTS PURGE;

CREATE TABLE participacio (
    idVotacio   INTEGER   CONSTRAINT FK1_participacioIdVotacio REFERENCES votacio(idVotacio) ON DELETE CASCADE,
    idCiutada   CHAR(32)  CONSTRAINT FK2_participacioIdCiutada REFERENCES ciutada(idCiutada) ON DELETE CASCADE,
    CONSTRAINT PK_participacio PRIMARY KEY(idVotacio, idCiutada)                                  
    );

--*************************************************************************
--  (ESTADISTIQUES) CREACIO STAT_ANY PER EMAGATZEMAR DADES ESTADISTIQUES PER ANY
--*************************************************************************

DROP TABLE stat_any CASCADE CONSTRAINTS PURGE;

CREATE TABLE stat_any (
    exercici          CHAR(4)       CONSTRAINT PK_stat_any PRIMARY KEY,   
    votacioMesPart    INTEGER       CONSTRAINT FK1_stat_anyVotacioMesPart REFERENCES votacio(idVotacio),
    paisMesVots       CHAR(2)       CONSTRAINT FK2_stat_anyPaisMesVots REFERENCES pais(idPais),
    regioMesVots      VARCHAR2(10)  ,
    localitatMesVots  VARCHAR2(10)  ,
    percenCiutNoVotat NUMBER(5,2)   CONSTRAINT CH_stat_anyPercenCiutNoVotat
                                              CHECK( percenCiutNoVotat BETWEEN 0 AND 100),
    CONSTRAINT FK3_stat_anyRegioMesVots FOREIGN KEY (paisMesVots, regioMesVots) REFERENCES regio(idPais, idRegio),
    CONSTRAINT FK4_stat_anyLocalitatMesVots FOREIGN KEY (paisMesVots, regioMesVots, localitatMesVots) REFERENCES localitat(idPais, idRegio, idLocalitat)
    );

--*************************************************************************
--  (ESTADISTIQUES) CREACIO STAT_VOTACIO PER EMAGATZEMAR DADES ESTADISTIQUES DE VOTACIONS
--*************************************************************************

DROP TABLE stat_votacio CASCADE CONSTRAINTS PURGE;

CREATE TABLE stat_votacio (
    idVotacioMesPart          INTEGER   CONSTRAINT FK1_stat_votidVotMesPart REFERENCES votacio(idVotacio),   
    idVotacioMesDifer         INTEGER   CONSTRAINT FK2_stat_votidVotMesDifer REFERENCES votacio(idVotacio),   
    idVotacioMenysDifer       INTEGER   CONSTRAINT FK3_stat_votidVotMenysDifer REFERENCES votacio(idVotacio),   
    nMaxVotacionsPersona      INTEGER,
    paisPersMaxVots           CHAR(2)   CONSTRAINT FK4_stat_votPaisPersMaxVots REFERENCES pais(idPais)
    );

--*************************************************************************
--  (ESTADISTIQUES) CREACIO STAT_PAIS_ANY PER EMAGATZEMAR DADES ESTADISTIQUES DE VOTACIONS PER PAISOS I ANYS
--*************************************************************************

DROP TABLE stat_pais_any CASCADE CONSTRAINTS PURGE;

CREATE TABLE stat_pais_any (
    idPais        CHAR(2)   CONSTRAINT FK1_stat_pais_anyIdPais    REFERENCES pais(idPais),   
    exercici      CHAR(4)   CONSTRAINT FK2_stat_pais_anyExercici  REFERENCES stat_any(exercici),   
    nVotacionsFi  INTEGER,   
    CONSTRAINT PK_stat_pais_any PRIMARY KEY(idPais, exercici)                                  
    );

--*************************************************************************
--  (ESTADISTIQUES) CREACIO STAT_CENS_ANY PER EMAGATZEMAR DADES ESTADISTIQUES DE VOTACIONS PER CENS I ANYS
--*************************************************************************

DROP TABLE stat_cens_any CASCADE CONSTRAINTS PURGE;

CREATE TABLE stat_cens_any (
    idCens          NUMBER      CONSTRAINT FK1_stat_cens_anyIdCens REFERENCES cens(idCens),   
    exercici        CHAR(4)     CONSTRAINT FK2_stat_cens_anyExercici  REFERENCES stat_any(exercici),   
    nVotacions      INTEGER,   
    valorMitjaPart  NUMBER(5,2) CONSTRAINT CH_stat_cens_anyValorMitjaPart
                                    CHECK( valorMitjaPart BETWEEN 0 AND 100),  
    CONSTRAINT PK_stat_cens_any PRIMARY KEY(idCens, exercici)                                  
    );

    
COMMIT
/